searchNodes=[{"type":"module","title":"FsBuild","doc":"A filesystem-based build engine which provides a flexible mechanism for parsing and processing files. This is a fork of NimblePublisher . Overview use FsBuild , from : Application . app_dir ( :app_name , &quot;priv/articles/**/*.md&quot; ) , adapter : { FsBuild.Adapters.MarkdownPublisher , [ ] } , build : Article , as : :articles The example above will: get all files whose path match the given pattern. extract data and metadata from files by using adapter FsBuild.Adapters.MarkdownPublisher . call Article.build/3 for each file, passing the data and metadata arguments. define a module attribute named @articles with all built articles returned by Article.build/3 function. Options :from - a wildcard pattern where to find all files. :adapter - the adapter and its options - {module(), any()} . :build - the name of the module that will build each file. :as - the name of the module attribute to store all built files. An example Let's build a blog by using the built-in adapter - FsBuild.Adapters.MarkdownPublisher , which has Markdown and code highlighting support. First, add fs_build and other required packages as dependencies: def deps do [ { :fs_build , &quot;~&gt; 1.0&quot; } , { :earmark , &quot;~&gt; 1.4&quot; } , { :makeup , &quot;~&gt; 1.0&quot; } , { :makeup_elixir , &quot;&gt;= 0.0.0&quot; } , { :makeup_erlang , &quot;&gt;= 0.0.0&quot; } ] end Each post stays in the priv/posts/ directory with the format: priv / posts / YEAR / MONTH - DAY - ID . md A typical post will look like this: # priv/posts/2020/04-17-hello-world.md %{ title : &quot;Hello world!&quot; , author : &quot;José Valim&quot; , tags : ~w(hello) , description : &quot;Let&#39;s learn how to say hello world&quot; } -- - This is the post . Then, we define a Post struct that expects all of the fields above. We will also have a :date field that we will build from the filename. Overall, it will look like this: defmodule MyApp.Blog.Post do @enforce_keys [ :id , :author , :title , :body , :description , :tags , :date ] defstruct [ :id , :author , :title , :body , :description , :tags , :date ] def build ( path , body , attrs ) do [ year , month_day_id ] = path |&gt; Path . rootname ( ) |&gt; Path . split ( ) |&gt; Enum . take ( - 2 ) [ month , day , id ] = String . split ( month_day_id , &quot;-&quot; , parts : 3 ) date = Date . from_iso8601! ( &quot; \#{ year } - \#{ month } - \#{ day } &quot; ) struct! ( __MODULE__ , [ id : id , date : date , body : body ] ++ Map . to_list ( attrs ) ) end end Now, we are ready to define our MyApp.Blog with FsBuild : defmodule MyApp.Blog do alias FsBuild.Adapters.MarkdownPublisher alias MyApp.Blog.Post use FsBuild , from : Application . app_dir ( :my_app , &quot;priv/posts/**/*.md&quot; ) , adapter : { MarkdownPublisher , highlighters : [ :makeup_elixir , :makeup_erlang ] } , build : Post , as : :posts # The @posts module attribute is first defined by FsBuild. # Let&#39;s further modify it by sorting all posts by descending date. @posts Enum . sort_by ( @posts , &amp; &amp;1 . date , { :desc , Date } ) # Let&#39;s also get all tags @tags @posts |&gt; Enum . flat_map ( &amp; &amp;1 . tags ) |&gt; Enum . uniq ( ) |&gt; Enum . sort ( ) # And finally export them def all_posts , do : @posts def all_tags , do : @tags end Important : Avoid injecting the @posts module attribute into multiple functions, as each call will make a complete copy of all posts. For example, if you want to show define recent_posts() as well as all_posts() , DO NOT do this: def all_posts , do : @posts def recent_posts , do : Enum . take ( @posts , 3 ) Instead do this: def all_posts , do : @posts def recent_posts , do : Enum . take ( all_posts ( ) , 3 ) Other helpers You may want to define other helpers to traverse your published resources. For example, if you want to get posts by ID or with a given tag, you can define additional functions as shown below: defmodule NotFoundError do defexception [ :message , plug_status : 404 ] end def get_post_by_id! ( id ) do Enum . find ( all_posts ( ) , &amp; ( &amp;1 . id == id ) ) || raise NotFoundError , &quot;post with id= \#{ id } not found&quot; end def get_posts_by_tag! ( tag ) do case Enum . filter ( all_posts ( ) , &amp; ( tag in &amp;1 . tags ) ) do [ ] -&gt; raise NotFoundError , &quot;posts with tag= \#{ tag } not found&quot; posts -&gt; posts end end Live reloading If you are using Phoenix, you can enable live reloading by simply telling Phoenix to watch the priv/posts/ directory. Open up config/dev.exs , search for live_reload: and add this to the list of patterns: live_reload : [ patterns : [ ... , ~r&quot;priv/posts/*/.*(md)$&quot; ] ] Custom adapters By using custom adapters, you can process any kind of files. For example, org files, JSON files, YAML files, etc. Checkout FsBuild.Adapter for more details.","ref":"FsBuild.html"},{"type":"behaviour","title":"FsBuild.Adapter","doc":"The specification of adapters.","ref":"FsBuild.Adapter.html"},{"type":"callback","title":"FsBuild.Adapter.init/1","doc":"Prepares for following works, such as: checking required dependencies starting necessary applications ...","ref":"FsBuild.Adapter.html#c:init/1"},{"type":"callback","title":"FsBuild.Adapter.transform/3","doc":"Transforms the content of files. It must return: a 2 element tuple with attributes and body - {data, metadata} a list of 2 element tuple with attributes and body - [{data, metadata} | _]","ref":"FsBuild.Adapter.html#c:transform/3"},{"type":"type","title":"FsBuild.Adapter.content/0","doc":"The content of file.","ref":"FsBuild.Adapter.html#t:content/0"},{"type":"type","title":"FsBuild.Adapter.data/0","doc":"The main data transformed from file.","ref":"FsBuild.Adapter.html#t:data/0"},{"type":"type","title":"FsBuild.Adapter.metadata/0","doc":"The metadata transformed from file.","ref":"FsBuild.Adapter.html#t:metadata/0"},{"type":"type","title":"FsBuild.Adapter.opts/0","doc":"The option for current adapter.","ref":"FsBuild.Adapter.html#t:opts/0"},{"type":"type","title":"FsBuild.Adapter.path/0","doc":"The path of file.","ref":"FsBuild.Adapter.html#t:path/0"},{"type":"module","title":"FsBuild.Adapters.MarkdownPublisher","doc":"An adapter for building a Markdown publishing system. Required dependencies :earmark :makeup Options :earmark_options - an %Earmark.Options{} struct :highlighters - which code highlighters to use. FsBuild uses Makeup for syntax highlighting and you will need to add its .css classes. You can generate the CSS classes by calling Makeup.stylesheet(:vim_style, &quot;makeup&quot;) inside iex -S mix. You can replace :vim_style by any style of your choice defined here .","ref":"FsBuild.Adapters.MarkdownPublisher.html"}]